/**********************************************************
 **********************************************************
 * Free/busy dialog functions.
 * 
 * Requires: freebusy-dialog.xul
 *           freebusy-profile.js
 *           freebusy-utils.js
 **********************************************************
 **********************************************************/
var freebusy = window.opener.com.dwise.freebusy;
var debug = window.opener.com.dwise.freebusy.utils.debug;
var gFreeBusyVersion = freebusy.gFreeBusyVersion;
var gProfCount = freebusy.gProfCount;
var gCalCount = freebusy.gCalCount;

var _LastSelProfile;
var _Profiles = new Object(); // profile map - local to dialog

function getSelectedCalendarNames() {
   // returns an array of the names of the selected calendars
   var selCals = new Array();

   var i = 0;
   while (1) {
      var myid = "cal" + i;
      var cal = document.getElementById(myid);
      if (cal == null)
         break;

      if (cal.checked == true) {
         debug("DIALOG: " + cal.label + " calendar is selected");
         selCals.push(cal.label);
      }
      i++;
   }

   return selCals;
}

/** ************************************************* */
/** ************************************************* */
/** EVENT FUNCTIONS FOR FREE/BUSY USER INTERFACE      */
/** ************************************************* */
/** ************************************************* */
function loadFreeBusyDialog() {
   try {
      debug("DIALOG: loadFreeBusyDialog");
      gCalCount = 0;

      // load the saved profiles into the drop list
      var fbProfile = document.getElementById("fbProfile");

      var firstProfile = null;
      for ( var pname in freebusy.gProfiles) {
         var p = freebusy.gProfiles[pname].copy(); // copy the profile object
                                                   // and save in map within
                                                   // dialog
         _Profiles[p.profileName] = p;

         debug("DIALOG: Adding profile to droplist: " + p.profileName);
         fbProfile.appendItem(p.profileName, p.profileName, "");

         if (firstProfile == null)
            firstProfile = p;
         fbProfile.selectedIndex = 0;
      }

      // load the existing calendars into the dialog
      var calendarManager = Components.classes["@mozilla.org/calendar/manager;1"].getService(Components.interfaces.calICalendarManager);

      var count = new Object();
      var calendarList = calendarManager.getCalendars(count);
      for ( var i in calendarList) {
         var calbox = document.getElementById("calbox");
         var myid = "cal" + i;

         var cal = document.createElement("checkbox");
         cal.setAttribute("id", myid);
         cal.setAttribute("label", calendarList[i].name);
         cal.setAttribute("checked", "false");
         calbox.appendChild(cal);

         gCalCount = (new Number(i)).valueOf() + 1;
      }

      if (firstProfile != null) {
         // select the first profile object in the list
         _LastSelProfile = firstProfile;
      }

      // set the initial data in the dialog and activate the controls
      changeProfile();
   } catch (exc) {
      debug("Exception: " + exc);
   }

   // show the version of the addon in the about panel
   var fbVer = document.getElementById("fbVersion");
   fbVer.value = "Version " + gFreeBusyVersion;

   var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
   var fbBranch = prefService.getBranch("com.dwise.thunderbird.freebusy.");
   // load the settings panel, there are four slots
   for ( var i = 1; i <= 4; i++) {
      var emailField = document.getElementById("contactEmail" + i);
      var urlField = document.getElementById("fbURL" + i);

      var emailMatch = fbBranch.getCharPref("emailMatch" + i);
      var urlPattern = fbBranch.getCharPref("urlPattern" + i);

      emailField.value = emailMatch;
      urlField.value = urlPattern;
   }

   var logFilePathField = document.getElementById("logFilePath");
   logFilePathField.value = freebusy.gLogFilePath;
   var enableLoggingField = document.getElementById("enableLogging");
   enableLoggingField.checked = freebusy.gEnableLogging;

   toggleLogging();
}

function toggleLogging() {
   var logFilePathField = document.getElementById("logFilePath");
   var enableLoggingField = document.getElementById("enableLogging");
   
   if (enableLoggingField.checked) {
      logFilePathField.disabled = false;
   }
   else {
      logFilePathField.disabled = true;
   }
}

function changePublishOpts(panel) {
   var deckid = document.getElementById("publishOpts");

   switch (panel) {
   case 'ftp':
      deckid.selectedIndex = 0;
      break;

   case 'localFile':
      deckid.selectedIndex = 1;
      break;
   }
}

function getSelectedProfile() {
   var fbProfile = document.getElementById("fbProfile");
   var selProfile = fbProfile.selectedItem;
   var profName = selProfile.value;

   return _Profiles[profName];
}

function createProfile() {
   var params = {
      inn : {
         name : "Main"
      },
      out : null
   };
   window.openDialog("chrome://freebusy/content/new-fbprofile-dialog.xul",
         "fbNewProfile", "chrome, dialog, modal, resizable=no", params).focus();

   debug("NEW-FB-PROFILE: Dialog closed.");
   if (params.out) {
      // User clicked ok. Process changed arguments; e.g. write them to disk or
      // whatever
      var fbProfile = document.getElementById("fbProfile");
      var existingProfile = _Profiles[params.out];
      
      if (existingProfile != null) {
         alert("A profile with the name '" + params.out + "' already exists.  Please choose another name.");
         return createProfile();
      }
      
      var p = new freebusy.profile.PublishProfile(params.out);
      debug("NEW-FB-PROFILE: adding new profile: " + p.profileName);

      fbProfile.appendItem(p.profileName, p.profileName, null);

      _Profiles[p.profileName] = p;
   } else {
      // User clicked cancel. Nothing is done here.
   }
}

function deleteProfile() {
   var fbProfile = document.getElementById("fbProfile");
   var secondItem = fbProfile.getItemAtIndex(1);
   if (secondItem == null) {
      alert("The Free/Busy addon must have at least one profile.  To delete this " +
            "profile, please first create a new profile and then delete this one.");
      return;
   }
   
   var selProfile = fbProfile.selectedItem;
   var indxItem = fbProfile.getIndexOfItem(selProfile);

   
   var profName = selProfile.value;
   //alert("Attempting to delete :" + profName);

   var profileNameList = window.opener.com.dwise.freebusy.utils.listProps(_Profiles);
   //alert("Before: " + profileNameList);
   delete _Profiles[profName];
   fbProfile.removeItemAt(indxItem);
   //alert("Delete was called.  Selecting first profile.");

   profileNameList = window.opener.com.dwise.freebusy.utils.listProps(_Profiles);
   //alert("After: " + profileNameList);
   
   fbProfile.selectedIndex = 0;
   changeProfile();
}

function changeProfile() {
   debug("DIALOG: changeProfile");
   // save the profile if we changed from an existing profile
   // get the profile object matching the selected droplist item
   var selProfile = getSelectedProfile();
   if (_LastSelProfile != selProfile) {
      _LastSelProfile.getDataFromDialog(document);
   }

   debug("DIALOG: selProfile: " + selProfile.profileName);

   selProfile.showDataInDialog(document, changePublishOpts);

   debug("DIALOG: activating controls");
   activateControls();

   _LastSelProfile = selProfile;
}

function activateControls() {
   // This function enables and disables controls based on whether a calendar is
   // selected
   var calList = document.getElementById("fbProfile");
   if (calList == null)
      return;

   var selCal = calList.selectedItem;
   if (selCal == null || selCal.value == "") {
      // disable all the controls
      document.getElementById("deleteProfile").disabled = true;
      document.getElementById("ftpRadio").disabled = true;
      document.getElementById("fileRadio").disabled = true;
      document.getElementById("ftpURL").disabled = true;
      document.getElementById("ftpUserid").disabled = true;
      document.getElementById("ftpPassword").disabled = true;
      document.getElementById("localFile").disabled = true;
      document.getElementById("pubNow").disabled = true;
      document.getElementById("autobox").disabled = true;
      document.getElementById("period").disabled = true;

      for ( var i = 0; i < gCalCount; i++) {
         document.getElementById("cal" + i).disabled = true;
      }
   } else {
      // enable all the controls
      document.getElementById("deleteProfile").disabled = false;
      document.getElementById("ftpRadio").disabled = false;
      document.getElementById("fileRadio").disabled = false;
      document.getElementById("ftpURL").disabled = false;
      document.getElementById("ftpUserid").disabled = false;
      document.getElementById("ftpPassword").disabled = false;
      document.getElementById("localFile").disabled = false;
      document.getElementById("pubNow").disabled = false;
      document.getElementById("autobox").disabled = false;
      document.getElementById("period").disabled = false;

      for ( var i = 0; i < gCalCount; i++) {
         document.getElementById("cal" + i).disabled = false;
      }
   }
}

function publishCalendar() {
   var profile = getSelectedProfile();
   profile.getDataFromDialog(document);
   freebusy.profile.doPublish(profile);
   alert("Publish complete.");
}

function doOK() {
   try {
      // save current screen data to object
      var selProfile = getSelectedProfile();
      selProfile.getDataFromDialog(document);

      var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      
      //need to loop through the previously stored profiles
      //and delete the ones no longer stored in the map of
      //profiles cached in the dialog data
      for ( var pname in freebusy.gProfiles) {
         var p = _Profiles[pname];
         if (p == null) {            
            var fbBranch = prefService.getBranch("com.dwise.thunderbird.freebusy." + pname + ".");
            fbBranch.deleteBranch("");            
         }
      }
      
      // save the profile map from the dialog back to the freebusy scope
      freebusy.gProfiles = _Profiles;

      // loop over set of profiles and save data to preferences storage
      for ( var pname in _Profiles) {
         var p = _Profiles[pname];
         debug("DIALOG: Saving data for profile: " + p.profileName);
         p.saveData();
         debug("DIALOG: Done saving data for profile: " + p.profileName);
      }

      freebusy.profile.saveProfileNames(_Profiles);      

      var fbBranch = prefService.getBranch("com.dwise.thunderbird.freebusy.");
      // save the settings panel, there are four slots
      for ( var i = 1; i <= 4; i++) {
         var emailField = document.getElementById("contactEmail" + i);
         var urlField = document.getElementById("fbURL" + i);

         fbBranch.setCharPref("emailMatch" + i, emailField.value);
         fbBranch.setCharPref("urlPattern" + i, urlField.value);
      }
      
      //save the log file path
      freebusy.gLogFilePath = document.getElementById("logFilePath").value;
      freebusy.gEnableLogging = document.getElementById("enableLogging").checked;
      fbBranch.setCharPref("logFilePath", freebusy.gLogFilePath);
      fbBranch.setBoolPref("enableLogging", freebusy.gEnableLogging);
      
      freebusy.gLoggingError = false; //reset flag that prevents excessive error message dialogs      
   } catch (exc) {
      alert('An error occurred while saving the FreeBusy dialog information.  The exception was: ' + exc);
      debug("DIALOG: Error saving dialog information: " + exc);
   }

   return true;
}
