/**********************************************************
 **********************************************************
 * Main entry functions and background thread code for 
 * Free/busy addon. This program defines a package for this 
 * code and a method to show the initial addon window/dialog.
 * In addition, it kicks off a timer that wakes up every 30
 * seconds to see if it is time to publish free/busy information.
 * 
 * Requires: freebusy-utils.js 
 *           freebusy-dialog.xul 
 *           freebusy-profile.js
 **********************************************************
 **********************************************************/

// create package structure if not existing
if (!com) var com = {};
if (!com.dwise) com.dwise = {};
if (!com.dwise.freebusy) com.dwise.freebusy = {};

// placeholders for subpackages
var utilpkg = com.dwise.freebusy.utils;
var profpkg = com.dwise.freebusy.profile;

com.dwise.freebusy = { // start package

   utils : utilpkg, // subpackage
   profile : profpkg, // subpackage

   /****************************************************************************
    * Package-level Data Members: Version number Map of profile objects Count of
    * profiles Count of available calendars, logFilePath
    ***************************************************************************/
   gFreeBusyVersion : "2.0",
   gProfiles : {},
   gProfCount : 0,
   gCalCount : 0,
   gEnableLogging : false,
   gLogFilePath : "", //populated below
   gLoggingError : false,

   FBProvider : function() {

   }

}; // end package

// Main routine starts here
with (com.dwise.freebusy) {
   //first thing to do is load the logging preferences
   var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
   var fbBranch = prefService.getBranch("com.dwise.thunderbird.freebusy.");

   try {
      var enableLogging = fbBranch.getBoolPref("enableLogging");
      if (enableLogging == true) {
         gEnableLogging = true;
      }
   }
   catch (e) {
      //no problem, let default values pass
   }

   try {   
      var logFilePath = fbBranch.getCharPref("logFilePath");
   }
   catch (e) {
      //no problem, let default values pass
   }
   
   if (logFilePath == undefined || logFilePath == null) {
      // Returns "WINNT" on Windows Vista, XP, 2000, and NT systems;  
      // "Linux" on GNU/Linux; and "Darwin" on Mac OS X.  
      var osString = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime).OS;  
      var userEnvironment = Components.classes["@mozilla.org/process/environment;1"].getService(Components.interfaces.nsIEnvironment);

      if (osString == "WINNT" || osString == "OS2") {
         try {
            logFilePath = userEnvironment.get("TEMP") + "\\freebusy.log";
         }
         catch (ex1) {
            alert("The TEMP environment variable is not set.  Assigning freebusy.log path to c:\\freebusy.log.");            
            logFilePath = "c:\\freebusy.log";
         }
      }
      else {
         try {
            logFilePath = userEnvironment.get("HOME") + "/.freebusy.log";
         }
         catch (ex2) {
            alert("The HOME environment variable is not set.  Assigning freebusy.log path to /tmp/freebusy.log.  This may collide on a multi-user system.");
            logFilePath = "/tmp/freebusy.log";
         }         
      }
   }
   gLogFilePath = logFilePath;
   
   utils.debug("FREEBUSY MAIN: Starting the FreeBusy extension scripts.");
   utils.debug("FREEBUSY MAIN: FreeBusy extension version: " + gFreeBusyVersion);
   utils.debug("FREEBUSY MAIN: Loading saved profiles into package scope.");

   gProfiles = new Object();
   gProfCount = profile.loadSavedProfiles(gProfiles);
   utils.debug("FREEBUSY MAIN: Loaded " + gProfCount + " profile(s).");

   for ( var profName in gProfiles) {
      utils.debug("FREEBUSY MAIN: Loaded profile " + profName);
      var p = gProfiles[profName];
      utils.debug("FREEBUSY MAIN: dump " + p.toString());
   }

   utils.debug("FREEBUSY MAIN: Starting a background thread for autopublishing.");
   setTimeout('com.dwise.freebusy.utils.timer();', 30000);  // 30 seconds from
                                                            // now, the
                                                            // background thread
                                                            // will check to see
                                                            // if it is time to
                                                            // publish
}

com.dwise.freebusy.showFreeBusyDialog = function() {
   var win = window.openDialog("chrome://freebusy/content/freebusy-dialog.xul",
         "fbDialog", "chrome, centerscreen");
   win.focus();
};
