/**********************************************************
 **********************************************************
 * Free/busy PublishProfile object definition.  This program
 * defines the data members, and all methods
 * belonging to package com.dwise.freebusy.profile and to
 * the class com.dwise.freebusy.profile.PublishProfile.
 * 
 * Requires: freebusy-utils.js
 *           freebusy-dialog.xul
 *           freebusy-profile.js
 **********************************************************
 **********************************************************/

// create package structure if not existing
if(!com) var com={};
if(!com.dwise) com.dwise={};
if(!com.dwise.freebusy) com.dwise.freebusy={};
if(!com.dwise.freebusy.profile) com.dwise.freebusy.profile={};

com.dwise.freebusy.profile = { // start package

packNameArray: function(nameArray)
{
    var nameString = "";
    for (i in nameArray)
    {
        nameString += nameArray[i].length;
        nameString += ":";
        nameString += nameArray[i];
        nameString += ":";        
    }
    
    return nameString;
},

unpackNameArray: function(nameString)
{
    var nameArray = new Array();
    
    while (nameString.length > 0)
    {
        var pos = nameString.indexOf(":");
        if (pos == -1)
        {
           nameArray.push(nameString);
           break;
        }
        var len = parseInt(nameString.substr(0, pos));
        var text = nameString.substr(pos + 1, len);
        nameArray.push(text);
        nameString = nameString.substr(pos + len + 2);
    }
    
    return nameArray;
},
		
/*******************************************************************************
 * *************************************************************** Operational
 * functions for parsing and publishing calendar data
 * ***************************************************************
 ******************************************************************************/
loadSavedProfiles: function(profileMap)
{
    var profCount = 0;
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var fbBranch = prefService.getBranch("com.dwise.thunderbird.freebusy.");
    var profNames = fbBranch.getCharPref("savedprofiles");
    
    if (profNames == null || profNames == "") profNames = "7:Default";
    
    com.dwise.freebusy.utils.debug("LOADSAVEDPROFILES: saveprofiles: " + profNames);
    
    var profNamesArray = this.unpackNameArray(profNames);

    for (var i in profNamesArray)
    {
    	com.dwise.freebusy.utils.debug("LOADSAVEDPROFILES:mark1");
        var profName = profNamesArray[i];
    	com.dwise.freebusy.utils.debug("LOADSAVEDPROFILES:mark2");
        var p = new this.PublishProfile(profName);
    	com.dwise.freebusy.utils.debug("LOADSAVEDPROFILES:mark3");
        profileMap[profName] = p;
    	com.dwise.freebusy.utils.debug("LOADSAVEDPROFILES:mark4");
        profCount++;
    	com.dwise.freebusy.utils.debug("LOADSAVEDPROFILES:mark5");
    }
    
    return profCount;
},

saveProfileNames: function(profileMap)
{
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var fbBranch = prefService.getBranch("com.dwise.thunderbird.freebusy.");

    var profNamesArray = new Array();
    for (var profName in profileMap)
    {
    	profNamesArray.push(profName);
    }
    
    var profNames = this.packNameArray(profNamesArray);

    com.dwise.freebusy.utils.debug("SAVEPROFILENAMES: saving these profile names: " + profNames);    
    fbBranch.setCharPref("savedprofiles", profNames);
},

doFTP: function(fbInfo, url)
{
	var stringStream = null;
    try
    {
	    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	    var uri = ioService.newURI(url,"UTF-8",null);
	    var channel = ioService.newChannelFromURI(uri).QueryInterface(Components.interfaces.nsIUploadChannel);

	    var unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
	    unicodeConverter.charset = "UTF-8";
	    fbInfo = unicodeConverter.ConvertFromUnicode(fbInfo) + unicodeConverter.Finish();

	    stringStream = Components.classes["@mozilla.org/io/string-input-stream;1"].createInstance(Components.interfaces.nsIStringInputStream);
	    stringStream.setData(fbInfo, -1); 

	    channel.setUploadStream(stringStream, "text/plain", -1);
	    channel.open();
    }
    catch (e)
    {
        com.dwise.freebusy.utils.debug(e);
        alert(e);
    }
    finally {
    	if (stringStream != null) {
    		stringStream.close();
    	    delete channel;
    	}
    }
},

doFile: function(fbInfo, fbFileName)
{
	var fbStream = null;
	
    try
    {
        var fbFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
        fbFile.initWithPath(fbFileName);

        var fbStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);

        fbStream.init(fbFile,
                      0x02 /* PR_WRONLY */ |
                      0x08 /* PR_CREATE_FILE */ |
                      0x20 /* PR_TRUNCATE */,
                      0700 /* read, write, execute/search by owner */,
                      0 /* unused */);

        fbStream.write(fbInfo, fbInfo.length);
    } catch (exc) {
        com.dwise.freebusy.utils.debug('file IO error' + exc);
        alert('file IO error' + exc);
    }
    finally {
    	if (fbStream != null) {
            fbStream.close();
    	}
    }
},

formatFreeBusy: function(stDate, enDate, debugTitle)
{
    var stdt = stDate.getInTimezone(UTC()).icalString;
    if (stDate.isDate) stdt += "T000000Z";
    var endt = enDate.getInTimezone(UTC()).icalString;		        
    if (enDate.isDate) endt += "T000000Z";
    return stdt + "/" + endt + debugTitle;                           
},

computeFreeBusyInfo: function(profile, calNames, fbInfoCallback)
{
    // creates a complete Free/Busy string for the calendars whose names
      // appear in calNames array
    try
    {
    	com.dwise.freebusy.utils.debug("COMPUTE: Computing the freebusy info for profile: " + profile);
        com.dwise.freebusy.utils.debug("COMPUTE: Computing the freebusy info for calNames: " + calNames);

        var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
        com.dwise.freebusy.utils.debug("COMPUTE: prefService: " + prefService);
        var idPrefStub = "mail.identity.id";
        var primaryEmail;
        for (var i = 0; i < 20; i++) 
        {
            try
            {
                var idBranch = prefService.getBranch(idPrefStub + i + ".");           
                primaryEmail = idBranch.getCharPref("useremail");
                break;
            }
            catch (exec)
            {
                // exception means that pref not found so loop will try again,
                  // up to 20 times
            }
        }
        
        com.dwise.freebusy.utils.debug("COMPUTE: primaryEmail: " + primaryEmail);

        var organizer = primaryEmail;
        // compute the min and max recur
        var dt = new Date();
        var year = 365 * 24 * 3600 * 1000; // milliseconds in a year
        var month = 31 * 24 * 2600 * 1000; // milliseconds in a month
        var minr = new Date(dt.valueOf() - month);
        var maxr = new Date(dt.valueOf() + year);
        com.dwise.freebusy.utils.debug("COMPUTE: minr: " + minr);
        com.dwise.freebusy.utils.debug("COMPUTE: maxr: " + maxr);
        var minrecur = minr.toLocaleFormat("%Y%m%d") + "T000000Z";
        var maxrecur = maxr.toLocaleFormat("%Y%m%d") + "T000000Z";
        com.dwise.freebusy.utils.debug("COMPUTE: createDateTime(minrecur): " + createDateTime(minrecur));
        com.dwise.freebusy.utils.debug("COMPUTE: createDateTime(minrecur): " + createDateTime(maxrecur));

        var fbstamp = now().getInTimezone(UTC()).icalString;

        var events = new Array();
        var strFreeBusy = "";
        var fbTimes = new Array();

        var calGetListener = {
        	calCount: 0,
            onOperationComplete: function(aCalendar, aStatus, aOperationType, aId, aDetail)
            {            
                com.dwise.freebusy.utils.debug("COMPUTE: Done collecting free/busy info for: " + aCalendar.name);
                
                for each (e in events) 
                {
                    if (e.getProperty("TRANSP") == "TRANSPARENT") { continue; }

                    var cutoff = createDateTime(minrecur);
                        
                    if (e.startDate)
                    {
                        if (e.recurrenceInfo != null)
                        {                  
                            var rstart = cutoff; 
                            var rend = createDateTime(maxrecur); 
                            var nCount = new Object();
                            var recurs = e.getOccurrencesBetween(rstart,rend,nCount);
                            for (var j = 0; j < nCount.value; j++)
                            {
                               //only add the fbtime interval if the end time is newer than the cutoff (one month ago)
                               if (recurs[j].endDate.compare(cutoff) > 0) {
                                  fbTimes.push(com.dwise.freebusy.profile.formatFreeBusy(recurs[j].startDate, recurs[j].endDate, ""));
                               }
                            }
                        }
                        else
                        {
                           //only add the fbtime interval if the end time is newer than the cutoff (one month ago)
                           if (e.endDate.compare(cutoff) > 0) {
                              fbTimes.push(com.dwise.freebusy.profile.formatFreeBusy(e.startDate, e.endDate, ""));                           
                           }
                        }
                    }
                }

                com.dwise.freebusy.utils.debug("COMPUTE: done looping through events");

                if (this.calCount > 1)
                {
                	this.calCount--;
                	return;
                }
                
                com.dwise.freebusy.utils.debug("COMPUTE: Looped through the last calendar.  Proceeding with publish.");
                
                fbTimes.sort();

                var fbmin;
                var fbmax;
                
                if (fbTimes.length > 0) {
                   fbmin = fbTimes[0];
                   fbmax = fbTimes[fbTimes.length - 1];

                   /*
                    * first half of the DT string (before the slash) is the start
                    * of the range
                    */
                   fbmin = fbmin.substr(0, fbmin.indexOf("/"));
                   /* now roll to start of the day */
                   fbmin = fbmin.substr(0, 9) + "000000Z";


                   /*
                    * second half of the DT string (after the slash) is the end
                    * of the range
                    */
                   fbmax = fbmax.substr(fbmax.indexOf("/")+1);
                   /* now roll to start of the day */
                   fbmax = fbmax.substr(0, 9) + "115959Z";
                }
                else {
                   fbmin = fbstamp;
                   fbmax = fbstamp;
                }

                // build string from array
                var fbStrings = "";

                for each (fb in fbTimes)
                {
                    fbStrings += "FREEBUSY:" + fb + "\n";
                }

                com.dwise.freebusy.utils.debug("COMPUTE: done computing free busy strings");

                strFreeBusy = "BEGIN:VCALENDAR\n" +
                              "PRODID:d-Wise Technologies, Inc. Free/Busy Thunderbird Extension - home brewed!\n" +
                              "VERSION:" + com.dwise.freebusy.gFreeBusyVersion + "\n" +
                              "METHOD:PUBLISH\n" +
                              "BEGIN:VFREEBUSY\n" +
                              "ORGANIZER:" + organizer + "\n" +
                              "DTSTAMP:" + fbstamp + "\n" +
                              "DTSTART:" + fbmin + "\n" +
                              "DTEND:" + fbmax + "\n" +
                              fbStrings +                     
                              "END:VFREEBUSY\n" +
                              "END:VCALENDAR\n";            

                com.dwise.freebusy.utils.debug("COMPUTE: returning strFreeBusy:\n" + strFreeBusy);
                fbInfoCallback(profile, strFreeBusy);
            },
            onGetResult: function(aCalendar, aStatus, aItemType, aDetail, aCount, aItems)
            {
                for each (item in aItems) {
                    events.push(item);
                }
            }
        };

        var calendarManager = Components.classes["@mozilla.org/calendar/manager;1"].getService(Components.interfaces.calICalendarManager);
        var count = new Object();
        var calendarList = calendarManager.getCalendars(count);
        var cal = calendarList[0];

        // set a counter for decrementing in the callback routine
        calGetListener.calCount = calNames.__count__;
        
        for (var i in calendarList)
        {
            var cal = calendarList[i];
            com.dwise.freebusy.utils.debug("COMPUTE: cal: " + cal.name);
            com.dwise.freebusy.utils.debug("COMPUTE: calNames[j]: " + calNames[j]);
            for (var j in calNames)
            {
                if (cal.name == calNames[j] || '*' == calNames[j])
                {
                    com.dwise.freebusy.utils.debug("COMPUTE: getting info for calendar: " + cal.name);
                    cal.getItems(Components.interfaces.calICalendar.ITEM_FILTER_ALL_ITEMS,
                                 0, null, null, calGetListener); 
                    com.dwise.freebusy.utils.debug("COMPUTE: returned from call to getItems");
                }
            }
        }

        com.dwise.freebusy.utils.debug("COMPUTE: waiting for cal items to be collected");
    }
    catch (e)
    {
        com.dwise.freebusy.utils.debug(e);
        alert(e);
    }    
},


doPublish: function(profile)
{    
    var calNames = this.unpackNameArray(profile.calendars);
    com.dwise.freebusy.utils.debug("PUBLISH: profile.calendars: " + profile.calendars);
    com.dwise.freebusy.utils.debug("PUBLISH: calNames: " + calNames);

    // This is asynchronous so plan on a call back once calendar info is
      // collected
    // Pass anonymous function to compute routine
    this.computeFreeBusyInfo(profile, calNames, function(profile, fbinfo) 
    {    
        com.dwise.freebusy.utils.debug("PUBLISH: profile.mode: " + profile.mode);    

        if (profile.mode == "file")
        {    
            var fbfile = profile.file_filename;
            com.dwise.freebusy.utils.debug("PUBLISH: publishing FB info to local file: " + fbfile);
            com.dwise.freebusy.profile.doFile(fbinfo, fbfile);
        }
        else if (profile.mode == "ftp")
        {
            var fbftp = profile.ftp_url;
            if (
                 (profile.ftp_userid != null && profile.ftp_userid != "") ||
                 (profile.ftp_password != null && profile.ftp_password != "")
               )
            {
            	// insert the username:password just before the hostname in the
               // ftp url
                var creds = profile.ftp_userid + ":" + profile.ftp_password + "@";
                var pos = fbftp.indexOf("//");
                if (pos > -1)
                {
                    fbftp = fbftp.substr(0, pos + 2) + creds + fbftp.substr(pos + 2);
                }
            }

            com.dwise.freebusy.utils.debug("PUBLISH: publishing FB info to ftp URL: " + fbftp);
            com.dwise.freebusy.profile.doFTP(fbinfo, fbftp);
        }
    });
},

/*******************************************************************************
 * ********************************************************* Class declaration
 * for PublishProfile *********************************************************
 ******************************************************************************/
PublishProfile: function(name)
{
    this.profileName = name;
    this.mode = "ftp";
    this.ftp_url = "ftp://ftp.company.com/freebusy/myuserid.ifb";
    this.ftp_userid = "anonymous";
    this.ftp_password = "nobody@nowhere.com";
    this.file_filename = "c:\\freebusy\\myuserid.ifb";
    this.calendars = "*";
    this.autopublish_enabled = "false";
    this.autopublish_period = "60";
    this.lastpublish = 0;
    this.loadData();
}

} // end package



/*******************************************************************************
 * ********************************************************* Class definition
 * for PublishProfile *********************************************************
 ******************************************************************************/
with (com.dwise.freebusy.profile) {
	
PublishProfile.prototype = {

    copy: function()
    {
	    // construct a copy - this will pull data from preferences storage
	    var pcopy = new PublishProfile(this.profileName);
	    // override with data from the instance
	    pcopy.mode = this.mode;
	    pcopy.ftp_url = this.ftp_url;
	    pcopy.ftp_userid = this.ftp_userid;
	    pcopy.ftp_password = this.ftp_password;
	    pcopy.ftp_filename = this.ftp_filename;
	    pcopy.calendars = this.calendars;
	    pcopy.autopublish_enabled = this.autopublish_enabled;
	    pcopy.autopublish_period = this.autopublish_period;
	    pcopy.lastpublish = this.lastpublish;
	    
	    return pcopy;
    },
		
    toString: function()
    {
        return "\n" +
        "======= Dump of PublishProfile object (" + this.profileName + ") =======\n" +
        "this.mode: " + this.mode + "\n" +
        "this.ftp_url: " + this.ftp_url + "\n" +
        "this.ftp_userid: " + this.ftp_userid + "\n" +
        "(length of)this.ftp_password: " + this.ftp_password.length + "\n" +
        "this.file_filename: " + this.file_filename + "\n" +
        "this.calendars: " + this.calendars + "\n" +
        "this.autopublish_enabled: " + this.autopublish_enabled + "\n" +
        "this.autopublish_period: " + this.autopublish_period + "\n" +
        "this.lastpublish: " + this.lastpublish + "\n" +
        "======= Dump of PublishProfile object ======="
        ;
    },
    
    loadData: function()
    {    
    	try 
    	{
           com.dwise.freebusy.utils.debug("LOADDATA: Loading data for profile: " + this.profileName);
           var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
           var fbBranch = prefService.getBranch("com.dwise.thunderbird.freebusy." + this.profileName + ".");
           // check for some data at this branch by counting child elements
           var nChildren = new Object();
           var children = fbBranch.getChildList("", nChildren);
           com.dwise.freebusy.utils.debug("LOADDATA: fbBranch: " + fbBranch + " nChildren: " + nChildren.value);
        
           if (fbBranch != undefined && nChildren.value > 0)
           {
              this.mode = fbBranch.getCharPref("mode");
              this.ftp_url = fbBranch.getCharPref("ftp.url");
              this.ftp_userid = fbBranch.getCharPref("ftp.userid");
        
              this.ftp_password = this.getPassword(this.ftp_url, this.ftp_userid);
        
              this.file_filename = fbBranch.getCharPref("file.filename");
              this.calendars = fbBranch.getCharPref("calendars");
              this.autopublish_enabled = fbBranch.getCharPref("autopublish.enabled");
              this.autopublish_period = fbBranch.getCharPref("autopublish.period");
              this.lastpublish = fbBranch.getIntPref("lastpublish");
           }
    	}
    	catch (exc)
    	{
           com.dwise.freebusy.utils.debug("LOADDATA: Exception loading data for profile: " + this.profileName + ": " + exc);
           
    	}
    },
    
    getPassword: function(url, userid) 
    {
    	var password = "";
    	try
    	{
       	    var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);

      	    var e = passwordManager.enumerator;
	    	while (e.hasMoreElements())
	    	{
	    	    try 
	    	    {
	    	        // get an nsIPassword object out of the password manager.
	    	        // This contains the actual password...
	    	        var passObject = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
	    	        if (passObject.host == url) {
	    	             // found it!
	    	             password = passObject.password;
	    	             break;
	    	        }
	    	    } catch (ex) {
	    	    	com.dwise.freebusy.utils.debug("GETPASSWORD: Exception getting passsword: " + ex);
	    	    }
	    	}
    	}
    	catch (exception)
    	{
            var myLoginManager = Components.classes["@mozilla.org/login-manager;1"].getService(Components.interfaces.nsILoginManager);  

            // Find users for the given parameters
            var logins = myLoginManager.findLogins({}, url, "", "FreeBusy");  

            // Find user from returned array of nsILoginInfo objects
            for (var i = 0; i < logins.length; i++) {  
                if (logins[i].username == userid) {  
                    password = logins[i].password;  
                    break;
                }
            }      		
    	}
    	
    	return password;
    },

    saveData: function()
    {
        com.dwise.freebusy.utils.debug("SAVEDATA: running saveData method for profile: " + this.profileName);
        var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
        var fbBranch = prefService.getBranch("com.dwise.thunderbird.freebusy." + this.profileName + ".");
        fbBranch.setCharPref("mode", this.mode);
        com.dwise.freebusy.utils.debug("SAVEDATA: saving data ftp.url: " + this.ftp_url);
        fbBranch.setCharPref("ftp.url", this.ftp_url);
        fbBranch.setCharPref("ftp.userid", this.ftp_userid);
        // disable legacy password saving to preferences - current code uses
         // password manager
        // fbBranch.setCharPref("ftp.password", this.ftp_password);
    	if (this.mode == 'ftp')
    	{
            com.dwise.freebusy.utils.debug("SAVEDATA: Attempting to use legacy PasswordManager.");
    		try
    		{
               var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager);
    	       passwordManager.addUser(this.ftp_url, this.ftp_userid, this.ftp_password);
    	       com.dwise.freebusy.utils.debug("SAVEDATA: Legacy password manager successful.");
    		}
        	catch (exception)
        	{
                com.dwise.freebusy.utils.debug("SAVEDATA: Legacy PasswordManager unsuccessful.  Attempting to use new LoginManager");
            	// Thunderbird3
                passwordManager = Components.classes["@mozilla.org/login-manager;1"].getService(Components.interfaces.nsILoginManager);
                var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",  
                                                             Components.interfaces.nsILoginInfo,  
                                                             "init");
                
                if (this.ftp_password != null && this.ftp_password != "")
                {
                    var loginInfo = new nsLoginInfo(this.ftp_url, null, "FreeBusy", this.ftp_userid, this.ftp_password, "", "");  
                    try {
                        passwordManager.addLogin(loginInfo);
                        com.dwise.freebusy.utils.debug("SAVEDATA: LoginManager save successful.");
                    }
                    catch (e)
                    {
                    	com.dwise.freebusy.utils.debug("SAVEDATA: e: " + e);
                    }
                }
                else
                {
                    com.dwise.freebusy.utils.debug("SAVEDATA: ftp password null or blank.  Saving not performed.");                	
                }
        	}
    	}

        fbBranch.setCharPref("file.filename", this.file_filename);
        fbBranch.setCharPref("calendars", this.calendars);
        fbBranch.setCharPref("autopublish.enabled", this.autopublish_enabled);
        com.dwise.freebusy.utils.debug("SAVEDATA: autopublish: " + this.autopublish_enabled);
        fbBranch.setCharPref("autopublish.period", this.autopublish_period);
        fbBranch.setIntPref("lastpublish", this.lastpublish);
        com.dwise.freebusy.utils.debug("SAVEDATA: done running saveData method");
    },
    
    showDataInDialog: function(doc, changePublishOpts)
    {
        com.dwise.freebusy.utils.debug("SHOWDATAINDIALOG: showing data for profile: " + this.profileName);
        com.dwise.freebusy.utils.debug("SHOWDATAINDIALOG: " + this.toString());
        var modeRadio = doc.getElementById("modeRadio");
        var ftpRadio = doc.getElementById("ftpRadio");
        var fileRadio = doc.getElementById("fileRadio");        
        var ftpURL = doc.getElementById("ftpURL");
        var ftpUserid = doc.getElementById("ftpUserid");
        var ftpPassword = doc.getElementById("ftpPassword");
        var localFile = doc.getElementById("localFile");
        var autobox = doc.getElementById("autobox");
        var period = doc.getElementById("period");

        ftpURL.value = this.ftp_url;
        ftpUserid.value = this.ftp_userid;
        ftpPassword.value = this.ftp_password;
        localFile.value = this.file_filename;
        
        if (this.mode == "ftp")
        {
            modeRadio.selectedItem = ftpRadio;
            changePublishOpts('ftp');
        }
        else if (this.mode == "file")
        {
            modeRadio.selectedItem = fileRadio;
            changePublishOpts('localFile');
        }

        if (this.autopublish_enabled == "true")
        {
            autobox.checked = true;
        }
        else
        {
            autobox.checked = false;
        }

        var persel = doc.getElementById("period" + this.autopublish_period);
        period.selectedItem = persel;
                
        // com.dwise.freebusy.utils.debug("this.calendars: " + this.calendars);
        
        // select calendars and deselect calendars
        var selectAllCalendars = false;
        var selectCals;
        if (this.calendars == "*")
        {
            selectAllCalendars = true;
            selectCals = new Array();
        }
        else
        {
            selectCals = com.dwise.freebusy.profile.unpackNameArray(this.calendars);
        }
        
        com.dwise.freebusy.utils.debug("SHOWDATAINDIALOG: selectAllCalendards: " + selectAllCalendars);
                
        var i = 0;                
        while (1)
	    {
	        var myid = "cal" + i;
            var cal = doc.getElementById(myid);
            if (cal == null) break;
            com.dwise.freebusy.utils.debug("SHOWDATAINDIALOG: calendar object " + i + ": " + cal);
            
            if (selectAllCalendars)
            {
                cal.checked = true;
            }
            else
            {
                cal.checked = false;
                for (var n in selectCals)
                {
                    if (selectCals[n] == cal.label)
                    {
                        cal.checked = true;
                    }
                }
            }            
            i++;
        }
    },
    
    getDataFromDialog: function(doc)
    {
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: start");
        var modeRadio = doc.getElementById("modeRadio");
        var ftpRadio = doc.getElementById("ftpRadio");
        var fileRadio = doc.getElementById("fileRadio");        
        var ftpURL = doc.getElementById("ftpURL");
        var ftpUserid = doc.getElementById("ftpUserid");
        var ftpPassword = doc.getElementById("ftpPassword");
        var localFile = doc.getElementById("localFile");
        var autobox = doc.getElementById("autobox");
        var period = doc.getElementById("period");

        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: modeRadio:" + modeRadio);
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: ftpRadio:" + ftpRadio);
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: fileRadio:" + fileRadio);
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: ftpURL:" + ftpURL);
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: ftpUserid:" + ftpUserid);
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: ftpPassword:" + ftpPassword);
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: localFile:" + localFile);
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: autobox:" + autobox);
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: period:" + period);

        var i = 0;
        var calArray = new Array();
        
        while (1)
        {
            var myid = "cal" + i;
            var cal = doc.getElementById(myid);
            if (cal == null) break;
            com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: calendar object " + i + ": " + cal);
            
            if (cal.checked == true)
            {
                calArray.push(cal.label);
            }
            i++;
        }
        
        this.calendars = com.dwise.freebusy.profile.packNameArray(calArray);
        
        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: calendars: " + this.calendars);

        this.ftp_url = ftpURL.value;
        this.ftp_userid = ftpUserid.value;
        this.ftp_password = ftpPassword.value;
        this.file_filename = localFile.value;
        
        if (modeRadio.selectedItem == ftpRadio)
        {
            this.mode = "ftp";
        }
        else if (modeRadio.selectedItem == fileRadio)
        {
            this.mode = "file";
        }

        if (autobox.checked == true)
        {
            this.autopublish_enabled = "true";
        }
        else
        {
            this.autopublish_enabled = "false";
        }

        this.autopublish_period = period.selectedItem.value;

        com.dwise.freebusy.utils.debug("GETDATAFROMDIALOG: finish");        
        com.dwise.freebusy.utils.debug(this.toString());
    }
}; // end object definition

} // end with block

