/**********************************************************
 **********************************************************
 * Free/busy calendar provider.  This code provides the 
 * busy time ranges to the Invite Attendees window in Thunderbird.
 * 
 * Requires: freebusy-utils.js
 **********************************************************
 **********************************************************/

//create package structure if not existing
if (!com) var com = {};
if (!com.dwise) com.dwise = {};
if (!com.dwise.freebusy) com.dwise.freebusy = {};

try {
   with (com.dwise.freebusy) {
      FBProvider.prototype = {

         readFBStream : function(url, aListener) {
            utils.debug("IFB PROVIDER: Beginning function readFBStream.");
            var fbData = "";            
            Components.utils.import("resource://gre/modules/NetUtil.jsm");
            
            if (NetUtil.readInputStreamToString) { 
               utils.debug("IFB PROVIDER: Detected app version more recent than 3.x.");

               //This is a newer Thunderbird so read using utility class
               var ioserv = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
               utils.debug("IFB PROVIDER: go ioserv.");
               var uri = ioserv.newURI(url, null, null);
               utils.debug("IFB PROVIDER: got uri.");
               var channel = ioserv.newChannelFromURI(uri);
               utils.debug("IFB PROVIDER: go channel.");
               var urlListener = {
                  onStartRequest : function(aRequest, aContext) {                 
                  },
                  
                  onDataAvailable : function(
                     aRequest,
                     aContext,
                     aInputStream,
                     aOffset,
                     aCount
                  ) {
                     utils.debug("IFB PROVIDER: reading input stream.");
                     fbData = NetUtil.readInputStreamToString(aInputStream, aInputStream.available());
                     utils.debug("IFB PROVIDER: input stream read complete.");
                     try {
                        utils.debug("IFB PROVIDER: calling function to process the FB stream data.");
                        aContext.myprovider.processFBStreamData(fbData, aListener);
                     }
                     catch (e) {
                        utils.debug(e);
                     }
                  },
                  
                  onStopRequest : function(aRequest, aContext, aStatusCode) {
                  },
                  
                  QueryInterface : function(aIID) {
                     if (!aIID.equals(Components.interfaces.nsISupports)
                         && !aIID.equals(Components.interfaces.nsIRequestObserver)) {
                        throw Components.results.NS_ERROR_NO_INTERFACE;
                     }
                     return this;
                  } 
               };

               utils.debug("IFB PROVIDER: opening channel.");
               channel.asyncOpen(urlListener, window);
            }
            else {
               utils.debug("IFB PROVIDER: Detected app version 3.x or older.");
               //This is an older Thunderbird so read using legacy mechanism
               // read the freebusy url for this user
               var channel;
               var stream;
               var cStream;
               try {
                  var ioserv = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
                  var channel = ioserv.newChannel(url, 0, null);
                  utils.debug("IFB PROVIDER: Opening FB url.");
                  var stream = channel.open();

                  if (channel instanceof Components.interfaces.nsIHttpChannel
                        && channel.responseStatus != 200) {
                     utils.debug("IFB PROVIDER: Channel response: " + channel.responseStatus);
                     return null;
                  }

                  utils.debug("IFB PROVIDER: Reading the stream.");
                  var cStream = Components.classes["@mozilla.org/intl/converter-input-stream;1"].createInstance(Components.interfaces.nsIConverterInputStream);
                  cStream.init(stream, "UTF-8", 0, 0);
                  cStream.QueryInterface(Components.interfaces.nsIUnicharLineInputStream);

                  if (cStream instanceof Components.interfaces.nsIUnicharLineInputStream) {
                     var line = {};
                     var moreLines = cStream.readLine(line);
                     if (!moreLines) {
                        utils.debug("IFB PROVIDER: Empty stream.  No free/busy data was read from url: " + url);
                        return null;                                                
                     }
                     
                     do { 
                        fbData += ("\n" + line.value);
                        moreLines = cStream.readLine(line);
                     } while (moreLines);
                  }
                  
                  this.processFBStreamData(fbData, aListener);
               }
               catch (e) {
                  utils.debug("IFB PROVIDER: Exception reading free/busy data from url: " + url);
                  utils.debug("Exception: " + e)
                  return null;                                                                  
               }
               finally {
                  utils.debug("IFB PROVIDER: Closing streams");
                  if (cStream) cStream.close();
                  if (stream) stream.close();
                  utils.debug("IFB PROVIDER: Closed");                  
               }
            }
         
            return null;
         },
         
         processFBStreamData : function(fbData, aListener) {
            var fbRanges = [];
            
            if (fbData != null && fbData.length > 0) {

               utils.debug("IFB PROVIDER: In processFBStreamData.  fbData: " + fbData);
               
               var rex = /FREEBUSY(?:.*;FBTYPE=[^;:]+)?.*:(.*)/g;
               var fbStrings = fbData.match(rex);
               
               utils.debug("IFB PROVIDER: Processing fbData.");

               if (fbStrings != null) {
                  for (w = 0; w < fbStrings.length; w++) {
                     var fbText = fbStrings[w];
                                    
                     var returnFBType = Components.interfaces.calIFreeBusyInterval.BUSY;                  
                     var regexp = /FBTYPE=([^;:]+)/;
                     var typedata = regexp.exec(fbText);
                     if (typedata != null && typedata.length > 1) {
                        var fbTypeString = typedata[1];
                        switch (fbTypeString) {
                           case "FREE":
                              returnFBType = Components.interfaces.calIFreeBusyInterval.FREE;
                              break;
                           case "BUSY-UNAVAILABLE":
                              returnFBType = Components.interfaces.calIFreeBusyInterval.BUSY_UNAVAILABLE;
                              break;
                           case "BUSY-TENTATIVE":
                              returnFBType = Components.interfaces.calIFreeBusyInterval.BUSY_TENTATIVE;
                              break;
                           default:
                              returnFBType = Components.interfaces.calIFreeBusyInterval.BUSY;
                        }
                     }
                     
                     var clazz = Components.classes["@mozilla.org/calendar/datetime;1"];
                     var iface = Components.interfaces.calIDateTime;
                     
                     var fbparse = /FREEBUSY(?:.*;FBTYPE=[^;:]+)?.*:(.*)/;
                     var tokens = fbparse.exec(fbText);
                     if (tokens != null && tokens.length > 1) {
                        var span = tokens[1].split("/");
                        var fbStart = clazz.createInstance(iface);
                        fbStart.icalString = span[0];
      
                        var fbEnd = clazz.createInstance(iface);
                        fbEnd.icalString = span[1];
      
                        var fbEntry = {
                           freeBusyType : returnFBType,
                           interval : {
                              start : fbStart,
                              end : fbEnd
                           }
                        };
      
                        fbRanges.push(fbEntry);                  
                     }
                  }
               }
               utils.debug("IFB PROVIDER: Done reading the lines.  Setting the result via callback.");
            }
            aListener.onResult(null, fbRanges);                     
            utils.debug("IFB PROVIDER: Result was set");            
         },
         
         getFreeBusyIntervals : function(aCalId, aRangeStart, aRangeEnd,
               aBusyTypes, aListener) {

            // create a ranges array to hold all of the free/busy times
            utils.debug("IFB PROVIDER: Starting provider.");

            utils.debug("IFB PROVIDER: Getting preferences.");
            var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
            var fbBranch = prefService.getBranch("com.dwise.thunderbird.freebusy.");

            // there are up to four FB url criteria that may need to be searched
            for ( var i = 1; i <= 4; i++) {
               utils.debug("IFB PROVIDER: Checking email and url pattern number: " + i);

               var emailMatch = fbBranch.getCharPref("emailMatch" + i);
               var urlPattern = fbBranch.getCharPref("urlPattern" + i);

               if (emailMatch == null || emailMatch == "" || urlPattern == null
                     || urlPattern == "") {
                  utils.debug("IFB PROVIDER: Iteration " + i
                        + " has missing values.  Skipping.");
                  continue;
               }

               var ematch = "^MAILTO:" + emailMatch.replace(".", "\\.", "g").replace("*", "(.*)", "g");
               utils.debug("IFB PROVIDER: Attempting to match against expression: " + ematch);
               var calex = new RegExp(ematch,"i"); /* /^MAILTO:(.*)@d-wise\.com/; */
               var result = calex.exec(aCalId);
               if (result == null) {
                  utils.debug("IFB PROVIDER: No match.");
                  continue;
               }

               utils.debug("IFB PROVIDER: Found a matching email address");
               utils.debug("IFB PROVIDER: Username is :" + result[1]);
               utils.debug("IFB PROVIDER: Url for free/busy is :" + url);

               var url = urlPattern.replace("*", result[1], "g");
               utils.debug("IFB PROVIDER: Url for free/busy is :" + url);

               // read the freebusy url for this user
               this.readFBStream(url, aListener);
            }
            utils.debug("IFB PROVIDER: concluding method");
            return null;
         },

         QueryInterface : function(aIID) {
            utils.debug("IFB PROVIDER: query interface: " + aIID);
            if (!aIID.equals(Components.interfaces.nsISupports)
                  && !aIID.equals(Components.interfaces.calIFreeBusyProvider)) {
               utils.debug("IFB PROVIDER: throwing no interface result");
               throw Components.results.NS_ERROR_NO_INTERFACE;
            }
            return this;
         }
      };
   }
   ;

   com.dwise.freebusy.utils.debug("IFB PROVIDER: getting calIFreeBusyService");
   var svc = Components.classes["@mozilla.org/calendar/freebusy-service;1"].getService(Components.interfaces.calIFreeBusyService);
   com.dwise.freebusy.utils.debug("IFB PROVIDER: got it");
   com.dwise.freebusy.utils.debug("IFB PROVIDER: adding provider");
   var myprovider = new com.dwise.freebusy.FBProvider(); 
   svc.addProvider(myprovider);
   com.dwise.freebusy.utils.debug("IFB PROVIDER: done");
} catch (e) {
   alert(e);
}
