/**********************************************************
 **********************************************************
 * Free/busy addon utility functions.
 * 
 * Requires: none 
 **********************************************************
 **********************************************************/

// create package structure if not existing
if (!com) var com = {};
if (!com.dwise) com.dwise = {};
if (!com.dwise.freebusy) com.dwise.freebusy = {};
if (!com.dwise.freebusy.utils) com.dwise.freebusy.utils = {};

com.dwise.freebusy.utils = { // start package

   /**
    * Appends a debug message to a log file.
    */
   debug : function(msg) {
   
   if (! com.dwise.freebusy.gEnableLogging || 
       com.dwise.freebusy.gLoggingError) {
      return;
   }
   
   // calc date time
   var dt = new Date();
   var dtmsg = dt.toString() + ":" + msg + "\n";

   
   // open log file
   try {
      var logFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
      logFile.initWithPath(com.dwise.freebusy.gLogFilePath);

      // if log file greater than 2MB chop to 5000 lines ~600KB
      if (logFile.exists() && logFile.fileSize > (1024 * 1024 * 2))
      {
         // figure out how to rewrite the file and keep only the last 5000 lines
         var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
         var cStream = Components.classes["@mozilla.org/intl/converter-input-stream;1"].createInstance(Components.interfaces.nsIConverterInputStream);

         inStream.init(logFile, 0x01 /* PR_RDONLY */,
                                0700 /* read, write, execute/search by owner */, 
                                0 /* unused */);
         buffer = {};
         cStream.init(inStream, "UTF-8", 0, 0);
         var bytesRead = 0;

         try {
            // read all but 600KB
            do {
               var nBytes = cStream.readString(8192, buffer);
               bytesRead += nBytes;
               if (bytesRead > (logFile.fileSize - 614400))
                  break;
            } while (nBytes);

            var logData = "";
            // now read the rest into a string in memory
            do {
               var nBytes = cStream.readString(8192, buffer);
               if (nBytes) {
                  logData += buffer.value; // slurp!
               }
            } while (nBytes);
         } finally {
            inStream.close();
            cStream.close();
         }

         // open the log file for output and truncate on opening
         var logStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);

         logStream.init(logFile,
                        0x02 /* PR_WRONLY */| 0x08 /* PR_CREATE_FILE */| 0x20 /* PR_TRUNCATE */,
                        0700 /* read, write, execute/search by owner */, 
                        0 /* unused */);

         // append log msg
         try {
            logStream.write(logData, logData.length);
         } finally {
            logStream.close();
         }
      }

      var logStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);

      logStream
            .init(
                  logFile,
                  0x02 /* PR_WRONLY */| 0x08 /* PR_CREATE_FILE */| 0x10 /* PR_APPEND */,
                  0700 /* read, write, execute/search by owner */, 0 /* unused */);

      // append log msg
      try {
         logStream.write(dtmsg, dtmsg.length);
      } finally {
         logStream.close();
      }
   } catch (exc) {
      alert('A file IO error was encountered writing the Free/Busy log file.  The exception was: ' + exc +
            ".  This could be due to enabling logging, but having an invalid log file path.  Try " +
            "unchecking the box for 'Enable logging' on the settings tab in the Free/Busy addon.");
      com.dwise.freebusy.gLoggingError = true; //set a flag to prevent excessive error message dialogs
   }

},

/**
 * Async timer function to perform periodic publishing of the saved publish
 * profiles.
 */
timer : function() {
   var profiles = com.dwise.freebusy.gProfiles;
   // function for automatic publishing of the calendar
   for ( var pname in profiles) {
      var p = profiles[pname];
      this.debug("AUTOPUBLISH: Checking profile " + p.profileName
            + " for auto-publishing.");

      if (p.autopublish_enabled == "true") {
         this.debug("AUTOPUBLISH: Need to publish " + p.profileName + " every " +
                    p.autopublish_period + " minutes.");
         // autopublish period is set to some number of minutes - so multiply by 60 seconds by
         // 1000 milliseconds to get proper offset

         var now = new Date();
         var pubDate = new Date(p.lastpublish);
         this.debug("AUTOPUBLISH: now: " + now + "  pubDate: " + pubDate);
         var due = new Date(p.lastpublish);
         due.setTime(due.getTime() +
                     (new Number(p.autopublish_period)).valueOf() * 60 * 1000);
         this.debug("AUTOPUBLISH: due: " + due);

         if (due < now) {
            this.debug("AUTOPUBLISH: It is time to publish now!");
            this.debug("AUTOPUBLISH: Time to publish " + p.profileName + ", Last published " + pubDate);
            this.debug("AUTOPUBLISH: and is requested to publish every " +
                       p.autopublish_period + " minutes.");
            this.debug("AUTOPUBLISH: publishing calendar now.")
            com.dwise.freebusy.profile.doPublish(p);
            this.debug("AUTOPUBLISH: done publishing calendar.")
            p.lastpublish = now;
         } else {
            this.debug("AUTOPUBLISH: Not time yet");
         }
      } else {
         this.debug("AUTOPUBLISH: Autopublish disabled for " + p.profileName);
      }
   }

   setTimeout('com.dwise.freebusy.utils.timer();', 60000); // check again in
   // one minute
},

/**
 * Object dumper method. Very useful when running in the extension developer
 * JavaScript window.
 */
listProps : function(obj) {
   var propnames = "";
   for ( var i in obj) {
      propnames += i + "\n";
   }
   return propnames;
}

}; // end package
